from typing import Dict, NamedTuple
from pymongo.database import Database
from .lazy_collection import LazyCollection


class LazyDatabase(NamedTuple):
    database: Database
    default_collection_name: str = None  # type: ignore

    def __getitem__(self, key: str):
        return LazyCollection(self.database[key])

    def find_one(
        self,
        collection: str = None,
        query: Dict = None,
        project: Dict = None,
    ):
        coll = self[collection or self.default_collection_name]

        return coll.find_one(query, project)

    def find(
        self,
        collection: str = None,  # type: ignore
        query: Dict = None,  # type: ignore
        project: Dict = None,  # type: ignore
    ):
        coll = self[collection or self.default_collection_name]

        return coll.find(query, project)

    def insert_one(
        self,
        collection: str = None,  # type: ignore
        document: Dict = None,  # type: ignore
    ):
        coll = self[collection or self.default_collection_name]

        return coll.insert_one(document)

    def update_set_one(
        self,
        collection: str = None,  # type: ignore
        filter: Dict = None,  # type: ignore
        document: Dict = None,  # type: ignore
    ):
        coll = self[collection or self.default_collection_name]

        return coll.update_set_one(filter, document)

    def count(
        self,
        collection: str = None,  # type: ignore
        query: Dict = None,  # type: ignore
    ):
        coll = self[collection or self.default_collection_name]

        return coll.count(query)

    def distinct(
        self,
        key: str,
        collection: str = None,  # type: ignore
    ):
        coll = self[collection or self.default_collection_name]

        return coll.distinct(key)



from .lazy_collection import LazyCollection
from .lazy_database import LazyDatabase
from .lazy_mongo import LazyMongo

from .insert_response import InsertResponse
from .update_response import UpdateResponse



from typing import Dict, NamedTuple
from pymongo.collection import Collection
from pymongo.errors import DuplicateKeyError
from .update_response import UpdateResponse
from .insert_response import InsertResponse


class LazyCollection(NamedTuple):
    collection: Collection

    def find_one(
        self,
        query: Dict = None,
        project: Dict = None,
    ):
        return self.collection.find_one(query, project)

    def find(
        self,
        query: Dict = None,  # type: ignore
        project: Dict = None,  # type: ignore
    ):
        return self.collection.find(query, project)

    def insert_one(
        self,
        document: Dict = None,  # type: ignore
    ):
        try:
            result = self.collection.insert_one(document)

            return InsertResponse(
                ok=True,
                result=result,
            )

        except DuplicateKeyError as e:
            return InsertResponse(
                ok=False,
                is_duplicate=True,
                error=e,
            )

        except Exception as e:
            return InsertResponse(
                ok=False,
                error=e,
            )

    def update_set_one(
        self,
        filter: Dict = None,  # type: ignore
        document: Dict = None,  # type: ignore
    ):
        try:
            result = self.collection.update_one(
                filter=filter,
                update={
                    "$set": document,
                },
                upsert=False,
            )

            return UpdateResponse(
                ok=True,
                result=result,
            )

        except DuplicateKeyError as e:
            return UpdateResponse(
                ok=False,
                is_duplicate=True,
                error=e,
            )

        except Exception as e:
            return UpdateResponse(
                ok=False,
                error=e,
            )

    def count(
        self,
        query: Dict = None,  # type: ignore
    ):
        return self.collection.count_documents(query)

    def distinct(self, key: str):  # type: ignore
        return self.collection.distinct(key)



from typing import Dict
from pymongo import MongoClient
from .lazy_database import LazyDatabase


class LazyMongo:
    def __init__(self):
        self.mongo: MongoClient = None  # type: ignore
        self.default_database: str = None  # type: ignore
        self.default_collection: str = None  # type: ignore

    def connect(self, uri: str):
        try:
            self.mongo = MongoClient(uri)
        except:
            pass

        return self

    def __getitem__(self, key: str):
        return LazyDatabase(
            database=self.mongo[key or self.default_database],
            default_collection_name=self.default_collection,
        )

    def find_one(
        self,
        database: str = None,
        collection: str = None,
        query: Dict = None,
        project: Dict = None,
    ):
        db = self[database or self.default_database]

        return db.find_one(collection, query, project)

    def find(
        self,
        database: str = None,  # type: ignore
        collection: str = None,  # type: ignore
        query: Dict = None,  # type: ignore
        project: Dict = None,  # type: ignore
    ):
        db = self[database or self.default_database]

        return db.find(collection, query, project)

    def insert_one(
        self,
        database: str = None,  # type: ignore
        collection: str = None,  # type: ignore
        document: Dict = None,  # type: ignore
    ):
        db = self[database or self.default_database]

        return db.insert_one(collection, document)

    def update_set_one(
        self,
        database: str = None,  # type: ignore
        collection: str = None,  # type: ignore
        filter: Dict = None,  # type: ignore
        document: Dict = None,  # type: ignore
    ):
        db = self[database or self.default_database]

        return db.update_set_one(
            collection,
            filter,
            document,
        )

    def count(
        self,
        database: str = None,  # type: ignore
        collection: str = None,  # type: ignore
        query: Dict = None,  # type: ignore
    ):
        db = self[database or self.default_database]

        return db.count(collection, query)

    def distinct(
        self,
        key: str,
        database: str = None,  # type: ignore
        collection: str = None,  # type: ignore
    ):
        db = self[database or self.default_database]

        return db.distinct(key, collection)



from typing import Dict, NamedTuple
from pymongo.collection import Collection
from pymongo.results import InsertOneResult


class InsertResponse(NamedTuple):
    ok: bool
    is_duplicate: bool = False
    result: InsertOneResult = None
    error: Exception = None



from typing import NamedTuple
from pymongo.results import UpdateResult


class UpdateResponse(NamedTuple):
    ok: bool
    is_duplicate: bool = False
    result: UpdateResult = None
    error: Exception = None
